﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using XNAWinformReversi.UI_Forms;

namespace XNAWinformReversi.GameLogic
{
    class AdvancedAI
    {
        enum Difficulty { Novice, Easy, Intermediate, Expert };
        enum Piece { White = 1, Black };
        private int _difficulty;
        private int playerColor;
        private int nextPlayer;
        private int searchDepth;
        private int moveValue;
        //private int cornerValue;
        //private int forfeitValue;
        //bool cornerMove;
        //private bool forfeit;
        private bool tookCorner;
        private MainUIForm _form;
        private BoardModel _model;

        public AdvancedAI(int color, int diffictuly, BoardModel model)
        {
            playerColor = color;
            _difficulty = diffictuly;
            _model = model;
        }

        private void setDifficulty()
        {
            if (_difficulty == (int)Difficulty.Novice)
            {
                searchDepth = 1;
                //forfeitValue = 1;
            }
            else if (_difficulty == (int)Difficulty.Easy)
            {
                searchDepth = 3;
                //forfeitValue = 2;
            }
            else if (_difficulty == (int)Difficulty.Intermediate)
            {
                searchDepth = 4;
                //forfeitValue = 4;
            }
            else if (_difficulty == (int)Difficulty.Expert)
            {
                searchDepth = 5;
                //forfeitValue = 6;
            }
        }

        private int changeColor(int color)
        {
            if (color == 1)
            {
                color = 2;
            }
            else
            {
                color = 1;
            }

            return color;
        }

        private bool checkGameOver(BoardModel board)
        {
            bool whiteCanMove = board.loadLegalMoves((int)Piece.White);
            bool blackCanMove = board.loadLegalMoves((int)Piece.Black);
            if (whiteCanMove == false && blackCanMove == false)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        
        private int calcScore(BoardModel board)
        {
            board.loadPieceCount();
            int score = board.getScore();

            // Increase the value of the move if it causes oppenent to skip a move
            //if (forfeit)
            //{
            //    forfeit = false;
            //    score += forfeitValue;
            //}

            return score;
        }

        private bool isCornerMove(int x, int y)
        {
            return (x == 0 && y == 0) || (x == 7 && y == 0) || (x == 0 && y == 7) || (x == 7 && y == 7);
        }

        public void getMove(MainUIForm form)
        {
            _form = form;

            _model.loadLegalMoves(playerColor);
            foreach (Square temp in _model.legalMovesList)
            {
                // If corner move is legal do it.
                if (isCornerMove(temp.x, temp.y))
                {
                    _form.startMove(playerColor, temp.x, temp.y);
                    tookCorner = true;
                    break;
                }
            }

            if (tookCorner == false)
            {
                setDifficulty();
                startGetBestMove(_model);
            }

            tookCorner = false;
        }

        public void startGetBestMove(BoardModel board)
        {
            //Tests.writeToLog("-----------------------------------------");
            //Tests.writeToLog("startGetBestMove: " + searchDepth + " " + playerColor + " " + int.MinValue + " " + int.MaxValue);
            //Tests.writeToLog("");

            getBestMove(board, searchDepth, playerColor, int.MinValue, int.MaxValue);
        }

        public int getBestMove(BoardModel board, int depth, int currentPlayer, int alpha, int beta)
        {

            // If the game is over, terminate the search
            if (checkGameOver(board))
            {

                int score = calcScore(board);

                // if player has won
                if (score > 0)
                    moveValue = 100;
                // if player has lost (or tied)
                else
                    moveValue = -100;

                //Tests.writeToLog("checkGameOver moveValue: " + moveValue);
                //Tests.writeToLog("");

                return moveValue;
            }

            // At the bottom of the search space
            if (depth == 0)
            {
                moveValue = calcScore(board);

                //Tests.writeToLog("Depth 0 moreValue: " + moveValue);
                //Tests.writeToLog("");

                return moveValue;
            }

            // If this player can't play, skip turn
            if (board.loadLegalMoves(currentPlayer) == false)
            {
                //forfeit = true;
                nextPlayer = changeColor(currentPlayer);
                moveValue = getBestMove(board, depth, nextPlayer, alpha, beta);
                return moveValue;
            }

            // Get the list of legal moves
            board.loadLegalMoves(currentPlayer);
            List<Square> moves = board.legalMovesList;

            //foreach (Square tempSq in moves)
            //{
            //    Tests.writeToLog("board moves: " + currentPlayer + " (" + tempSq.x + "," + tempSq.y + ")");
            //}
            //Tests.writeToLog("");

            // Find the best currentMove
            Square bestMove = Square.Empty;
            foreach (Square currentMove in moves)
            {
                // Copy the board
                BoardModel currentBoard = ObjectCopier.Clone(board);

                //Tests.objectCopierTest(board, currentBoard);

                // Make the selected move
                currentBoard.loadPiecesToFlip(currentPlayer, currentMove.x, currentMove.y);
                currentBoard.setSquareValue(currentPlayer, currentMove.x, currentMove.y);

                //Tests.writeToLog("setSquareValue: " + currentPlayer + "(" + currentMove.x + "," + currentMove.y + ")");
                //Tests.writeToLog("");               

                foreach (Square flipSquare in currentBoard.piecesToFlipList)
                {
                    currentBoard.setSquareValue(currentPlayer, flipSquare.x, flipSquare.y);

                    //Tests.writeToLog("flipSquare: " + currentPlayer + " (" + flipSquare.x + "," + flipSquare.y + ")");
                }
                //Tests.writeToLog("");

                // Evaluate the board
                nextPlayer = changeColor(currentPlayer);
                int currentValue = getBestMove(currentBoard, depth - 1, nextPlayer, alpha, beta);

                //Tests.writeToLog("getBestMove Returned currentValue " + currentValue);
                //Tests.writeToLog("");

                // MAX
                if (currentPlayer == playerColor)
                {
                    if (currentValue > alpha)
                    {
                        alpha = currentValue;
                        bestMove = currentMove;
                    }
                    // Prune
                    if (alpha >= beta)
                    {
                        //Tests.writeToLog("Alpha " + alpha);
                        //Tests.writeToLog("");

                        return alpha;          
                    }
                }
                // MIN
                else
                {
                    if (currentValue < beta)
                    {
                        beta = currentValue;
                        bestMove = currentMove;
                    }
                    // Prune
                    if (alpha >= beta)
                    {
                        //Tests.writeToLog("Beta " + beta);
                        //Tests.writeToLog("");

                        return beta;
                    }
                }
            }

            // Make the best move
            if (!bestMove.Equals(Square.Empty))
                _form.startMove(currentPlayer, bestMove.x, bestMove.y);

            // Return the value of the best move
            if (currentPlayer == playerColor)
            {
                return alpha;
            }
            else
            {
                return beta;
            }
        }
    }
}
