﻿using System;
using System.Collections.Generic;

namespace ReversiV2
{
    public class EasyAI
    {

        enum Piece { White = 1, Black };
        private int _playingColor;
        private BoardModel _model;


        public EasyAI(int color, BoardModel model)
        {
            _playingColor = color;
            _model = model;
        }

        public Square getComputerMove()
        {
            // basic AI chooses move that flips most pieces
            // need to add in some heuristics for corner, near corner, safe pieces etc.
            _model.loadLegalMoves(_playingColor);
            _model.legalMovesList.Shuffle();
            int bestScore = -1;
            Square bestMove = null;  

            foreach (Square temp in _model.legalMovesList)
            {
                // if corner move is legal do it.
                if(isCornerMove(temp.x, temp.y))
                {
                    return temp;
                }
                // else do move that flips the most pieces.
                _model.loadPiecesToFlip(_playingColor, temp.x, temp.y);
                if (_model.piecesToFlipList.Count > bestScore)
                {
                    bestMove = temp;
                    bestScore = _model.piecesToFlipList.Count;
                }
            }
            return bestMove;
        }

        private bool isCornerMove(int x, int y)
        {
            return (x == 0 && y == 0) || (x == 7 && y == 0) || (x == 0 && y == 7) || (x == 7 && y == 7);
        }

    }

    static class MyExtensions
    {
        // standard Fisher-Yates shuffle example from StackOverflow.
        static readonly Random Random = new Random();
        public static void Shuffle<T>(this IList<T> list)
        {
            int n = list.Count;
            while (n > 1)
            {
                n--;
                int k = Random.Next(n + 1);
                T value = list[k];
                list[k] = list[n];
                list[n] = value;
            }
        }
    }
}
