﻿using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;

namespace ReversiV2
{
    public class PlayerVsAIController
    {
        enum Piece { White = 1, Black };
        private int _computerColor;
        private int _playerColor;
        private BoardModel _model;
        private EasyAI _computerAI;
        private ReversiUI _form;
        private bool _playerCanMove = false;
        private bool _computerCanMove = false;
        public BoardStateList _boardStateList { get; set; }

        // to be removed
        private int tempGameMoves = 0;  // temp holder to simulate no gameMoves in boardStateList

        public PlayerVsAIController(ReversiUI form, BoardModel model, int AIColor)
        {
            // setup variables and AI instance.
            _form = form;
            _model = model;
            _computerColor = AIColor;
            _playerColor = _model.getOpponentColor(AIColor);
            _computerAI = new EasyAI(_computerColor, _model);

            //------------Under Construction-------------------
            // Setup New Game
            //_model.newGame();
            //continueGame();

        }

        public void continueGame()
        {

            _form.setPieceCounts();
            notifyWhosTurn();

            // if it is currently the players turn
            if (_model.getCurrentPlayer() == _playerColor)
            {
                _playerCanMove = _model.loadLegalMoves(_playerColor);
                // if ^ true game flow stalls here until user clicks on a valid move and handleMouseClick() fires.

                // if player cannot move; note it and change currentPlayer.
                if (!_playerCanMove)
                {
                    _playerCanMove = false;
                    _model.setCurrentPlayer(_computerColor);
                    tempGameMoves++;  // temp holder until game boardState list is created.
                    _form.setMessage("You Cannot move. Computer goes again");
                    checkGameOver();
                }
            }
            // if it is the computers move.
            else if(_model.getCurrentPlayer() == _computerColor)
            {

                _computerCanMove = _model.loadLegalMoves(_computerColor);

                if (_computerCanMove)
                {
                    Square temp = _computerAI.getComputerMove();
                    _model.loadPiecesToFlip(_computerColor, temp.x, temp.y);
                    _model.setSquareValue(_computerColor, temp.x, temp.y);
                    _boardStateList.add(_model._board2DArray, _model.getCurrentPlayer());
                    _form.setPlacedPieceRect(temp.x, temp.y);
                    _form.doPieceFlips();
                }
                else if (!_computerCanMove)
                {
                    _computerCanMove = false;
                    _model.setCurrentPlayer(_playerColor);
                    tempGameMoves++;  // temp holder until game boardState list is created.
                    _form.setMessage("Computer Cannot Move. Your turn again.");
                    checkGameOver();
                }

            }

        }

        public void handleMouseClicks(int x, int y)
        {
            if (_model.getCurrentPlayer() == _playerColor)
            {
                foreach (Square temp in _model.legalMovesList)
                {
                    if (temp.x == x && temp.y == y)
                    {
                        _model.loadPiecesToFlip(_playerColor, x, y);
                        _model.setSquareValue(_playerColor, x, y);
                        _boardStateList.add(_model._board2DArray, _model.getCurrentPlayer());
                        _form.setPlacedPieceRect(temp.x, temp.y);
                        _form.Invalidate();
                        _form.doPieceFlips();

                        break;
                         
                    }
                }
            }
        }

        public void doNewGame()
        {
            _form.setPieceCounts();
            _form.setMessage("Black Moves First");
            _form.gameInProgress = true;
            _boardStateList = new BoardStateList();
            continueGame();
        }

        public void doSaveGame()
        {
            IFormatter formatter = new BinaryFormatter();
            Stream stream = new FileStream("testSave.bin", FileMode.Create, FileAccess.Write,FileShare.None);
            formatter.Serialize(stream, _boardStateList);
            stream.Close();

        }

        public void doLoadGame()
        {
            IFormatter formatter = new BinaryFormatter();
            Stream stream = new FileStream("testSave.bin", FileMode.Open, FileAccess.Read, FileShare.Read);
            BoardStateList list = (BoardStateList)formatter.Deserialize(stream);

            stream.Close();
        }


        private void doGameOver()
        {
            int winner = _model.getWinner();
            if (winner == 0)
            {
                _form.setMessage("--- GAME is a Tie ---");
            }
            else if (winner == _computerColor)
            {
                _form.setMessage("--- Computer Wins ---");
            }
            else
            {
                _form.setMessage("--- You Win ---");
            }

            _form.gameInProgress = false;

        }

        private void notifyWhosTurn()
        {
            if (_model.getCurrentPlayer() == _computerColor)
            {
                _form.setMessage("--- Computer's Move ---");
            }
            else
            {
                _form.setMessage("--- Your Move ---");
            }
        }

        private void checkGameOver()
        {
            // if both player and computer cannot move then game over.
            if (!_playerCanMove && !_computerCanMove)
            {
                doGameOver();
            }
            else
            {
                continueGame();
            }
        }


    }
}
