﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace ReversiV2
{
    public partial class ReversiUI : Form
    {
        // setup some drawing tools
        Pen blackPen = new Pen(Color.Black, 2);
        Pen drkGreenPen = new Pen(Color.DarkGreen, 2);
        Pen redPen = new Pen(Color.Red, 3);
        SolidBrush greenBrush = new SolidBrush(Color.LightGreen);
        SolidBrush blackBrush = new SolidBrush(Color.Black);
        SolidBrush whiteBrush = new SolidBrush(Color.White);

        // declare variables for UI
        enum Piece { White = 1, Black };
        private int boardWidth;
        private int sqrSize;
        private int xOffset;
        private int yOffset;
        private Rectangle piecePlacedRect;
        public bool drawPiecePlaced = false;
        private BoardModel model;
        private PlayerVsAIController controller;

        // testing area
        private int count3 = 0;
        private int count2 = 0;
        private int indexToPop = 0;
        private int numOfFlips = 0;
        private int flipColor;
        private int flipX;
        private int flipY;
        private Rectangle flipRect;
        private bool mouseClicksDisabled = false;
        public bool gameInProgress = false;

        public ReversiUI()
        {
            InitializeComponent();

            // Initialize UI variables
            xOffset = 50;
            yOffset = 100;
            boardWidth = 400;
            sqrSize = boardWidth / 8;
            SetStyle(ControlStyles.UserPaint, true);
            //SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            SetStyle(ControlStyles.OptimizedDoubleBuffer, true);

            //-----------------------------------------------------------
            //------------Under Construction------------------------------
            model = new BoardModel();
            model.newGame();
            blackPieceCountLabel.Text = "";
            whitePieceCountLabel.Text = "";
            messageLabel.Text = "Welcome To ReversiV2";
   
        }

        private void ReversiUI_Paint(object sender, PaintEventArgs e)
        {
            // NOTE: All these coordinates are a mess.....

            e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            // make sure to set double buffered to true/ add SetStyle lines. 

            // draw Reversi Board
            e.Graphics.FillRectangle(greenBrush, xOffset, yOffset, boardWidth, boardWidth);
            for (int i = 0; i <= boardWidth; i += sqrSize)
            {
                // horizontal lines
                e.Graphics.DrawLine(drkGreenPen, xOffset, yOffset+i, xOffset+boardWidth, yOffset+i);
                // vertical lines
                e.Graphics.DrawLine(drkGreenPen, i+xOffset, yOffset, i+xOffset, boardWidth+yOffset);
            }
            e.Graphics.DrawRectangle(blackPen, xOffset, yOffset, boardWidth, boardWidth);

            // draw board state
            // Note: to be able to resize board need to compute the 40, 5, etc...
            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    int temp = model.getSquareValue(x, y);
                    if ( temp == (int)Piece.Black)
                    {
                        e.Graphics.FillEllipse(blackBrush, (x*sqrSize)+ xOffset + 5, 
                            (y*sqrSize) + yOffset + 5, 40, 40);
                    }
                    else if (temp == (int)Piece.White)
                    {
                        e.Graphics.FillEllipse(whiteBrush, (x * sqrSize) + xOffset + 5, 
                            (y * sqrSize) + yOffset + 5, 40, 40);
                    }
                }
            }

            // draw piece placed if currently doing flip animation.
            if (drawPiecePlaced)
            {
                e.Graphics.DrawEllipse(redPen, piecePlacedRect);
            }

            // section for animation flips
            if (flipColor == (int)Piece.White)
            {
                if (count2 > 20)
                {
                    e.Graphics.FillEllipse(blackBrush, flipRect);
                }
                else
                {
                    e.Graphics.FillEllipse(whiteBrush, flipRect);
                }
            }
            else if (flipColor == (int)Piece.Black)
            {
                if (count2 > 20)
                {
                    e.Graphics.FillEllipse(whiteBrush, flipRect);
                }
                else
                {
                    e.Graphics.FillEllipse(blackBrush, flipRect);
                }
            }
        } 

        public void setPlacedPieceRect(int x, int y)
        {
            drawPiecePlaced = true;
            piecePlacedRect = new Rectangle((x*sqrSize)+ xOffset + 5, (y*sqrSize) + yOffset + 5, 40, 40);
            
        }

        private void ReversiUI_MouseClick(object sender, MouseEventArgs e)
        {
            if (!mouseClicksDisabled)
            {
                int clickedX = (e.X) / sqrSize;
                int clickedY = (e.Y) / sqrSize;
                int x = clickedX - 1; // note this fool; dont get yerself a one off error again
                int y = clickedY - 2; // note this fool; dont get yerself a one off error AGAIN
                //label1.Text = "X: " + x.ToString() + ", Y: " + y.ToString();

                //------------Under Construction-------------------
                if (gameInProgress)
                {
                    controller.handleMouseClicks(x, y);
                }
            }
        }

        public void doPieceFlips()
        {
            mouseClicksDisabled = true;
            count2 = 0;
            indexToPop = 0;
            numOfFlips = model.piecesToFlipList.Count;
            setFlipRect();
            animationTimer.Start();
        }

        private void setFlipRect()
        {
            Square temp = model.piecesToFlipList[indexToPop];
            flipX = temp.x;
            flipY = temp.y;
            int opponent = model.getOpponentColor(model.getCurrentPlayer());
            flipColor = opponent;
            model.setSquareValue(0, flipX, flipY);
            //this.Invalidate();
        }

        public void setMessage(string str)
        {
            messageLabel.Text = str;
            //this.Invalidate();
        }

        public void setPieceCounts()
        {
            model.loadPieceCount();
            blackPieceCountLabel.Text = "Black: " + model.getBlackPieceCount();
            whitePieceCountLabel.Text = "White: " + model.getWhitePieceCount();
        }

        private void animationTimer_Tick(object sender, EventArgs e)
        {
            
            flipRect = new Rectangle((flipX * sqrSize) + xOffset + 5, (flipY * sqrSize) + yOffset + 5 + count2, 40, 40 - (count2 * 2));
            this.Invalidate();
            count2++;
            //label1.Text = Convert.ToString(count2);
            if (count2 > 40)
            {
                model.setSquareValue(model.getCurrentPlayer(), flipX, flipY);
                indexToPop++;
                if (indexToPop < numOfFlips)
                {
                    //this.Invalidate();
                    count2 = 0;
                    setFlipRect();
                }
                else
                {
                    int opponent = model.getOpponentColor(model.getCurrentPlayer());
                    animationTimer.Stop();
                    drawPiecePlaced = false;
                    model.setCurrentPlayer(opponent);
                    setPieceCounts();
                    count3 = 0;
                    playerChangeTimer.Start();
                }
                
            }
        }

        private void playerChangeTimer_Tick(object sender, EventArgs e)
        {
            // Waits a few ticks before doing next players turn.
            count3++;
            if (count3 > 8)
            {
                playerChangeTimer.Stop();
                messageLabel.Text = "";
                mouseClicksDisabled = false;
                controller.continueGame();
            }
        }

        private void newGameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // need to poll whatever control for who is playing what color, p v p, p v AI, etc
            // if game in progress pop up window to ask are you sure?? exit this game
            // or disable until resign game is hit...

            //------------Under Construction-------------------
            controller = new PlayerVsAIController(this, model, (int)Piece.White);
            controller.doNewGame();
        }

        private void saveGameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            controller.doSaveGame();
        }

        private void loadGameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            controller.doLoadGame();
        }

        private void undoMoveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // pass
        }
    }
}
